// File Timer.h for CC1B
#ifndef _TIMER_H_
#define _TIMER_H_
//include other libraries here
#include <system/memory.h>
#pragma library 1

/**
 * \file Timer.h
 * \brief Library for timer support
 *
 * IMPORTANT: This library must be included after the library Task.h if that library is also used.
 *
 * This library provides functions to support 8/16/24/32bit timers. It declares a single free running
 * timer that is incremented every interrupt cycle. The application must define TIMER_DATA to set the start
 * address of the free running timer data area.
 * The application can declare (local) Timer8, Timer16, Timer24 and Timer32 objects to have multiple
 * independant timers.
 *
 * There are seven public functions provided:
 * - Timer_timer() gets the current free running timer count value.
 * - Timer_passedticks() gets the number of ticks passed since the last call to Timer_mark().
 * - Timer_timeout() checks wether the Timer object has expired.
 * - Timer_mark() marks the Timer object with an expiration value in ticks.
 * - Timer_markUSEC() marks the Timer object with an expiration value in microseconds.
 * - Timer_markMSEC() marks the Timer object with an expiration value in milliseconds.
 * - Timer_markSEC() marks the Timer object with an expiration value in seconds.
 *
 * The Timer object time unit is a tick. Each tick represents (2^shift)*INTPERIOD/CPUFREQ seconds
 * where shift is the number of bits the free running timer count is rightshifted. This shift allows the
 * use of a high resolution free running timer, to provide a large roundtrip time, and small Timer objects
 * that conserve memory but are able to have large expiration times (with reduced resolution).
 *
 * The functions Timer_markUSEC(), Timer_markMSEC() and Timer_markSEC() select the minimal shift value required
 * to get a specific timeout in microseconds, milliseconds or seconds. This is real time and the returned value
 * depends on used CPUFREQ and INTPERIOD settings. Basically these functions hide all
 * the details regarding ticks. If the requested timeout cannot be generated for the used Timer object
 * and/or used CPUFREQ and INTPERIOD, then an error is generated.
 *
 * Before including this library three parameters must be defined:
 * - TIMER_DATA which defines the address of the free running timer data area.
 * - TIMER_RUNxx where xx can be 08, 16, 24 or 32. This sets the free running timer resolution.
 * - TIMER_USEyy where yy can be 08, 16, 24 or 32. This sets the highest Timer object resolution.
 *
 * Omitting TIMER_RUNxx selects TIMER_RUN08. Omitting TIMER_USEyy selects TIMER_USE08.
 * Note that yy can never be larger than xx, even if defined as such.
 *
 * TIMER_RUN32 properties:
 * - data area 10 bytes
 * - free running timer resolution 32bits
 * - roundtrip time (2^32)*(INTPERIOD/CPUFREQ) seconds
 *
 * TIMER_RUN24 properties:
 * - data area 8 bytes
 * - free running timer resolution 24bits
 * - roundtrip time (2^24)*(INTPERIOD/CPUFREQ) seconds
 *
 * TIMER_RUN16 properties:
 * - data area 6 bytes
 * - free running timer resolution 16bits
 * - roundtrip time (2^16)*(INTPERIOD/CPUFREQ) seconds
 *
 * TIMER_RUN08 properties:
 * - data area 4 bytes
 * - free running timer resolution 8bits
 * - roundtrip time (2^8)*(INTPERIOD/CPUFREQ) seconds
 *
 * The difference between TIMER_RUNxx and TIMER_USEyy is best illustrated with an example.
 * In this example CPUFREQ is 20MHz and INTPERIOD is 174.
 *
 * Suppose you want to have a timer that can expire in 10 seconds. For the given CPUFREQ and INTPERIOD
 * that requires 10/(174/20_000_000) counts is 1149425 counts. This can be done with a free running timer
 * of 24 bits. So you select TIMER_RUN24.
 *
 * 1149425 requires 21 bits (2^21 = 2097152). To save memory you can declare a shift parameter for
 * Timer objects. If you don't need much resolution you can use a Timer8 object with a shift value of
 * 13 to 16. The upper limit calculates from 24 (free running timer resolution) - 8 (Timer object resolution).
 * The lower limit calculates from 21 (minimal required bits) - 8 (Timer object resolution).
 *
 * If you select 13, then each count in the Timer8 object represents (2^13)*(174/20_000_000) = 0.0712704 seconds.
 * The total range for the Timer8 object is (2^8)*0.0712704 = 18.2452224 seconds.
 * The 10 second timeout value is 10/0.0712704 = 140.
 *
 * If you select 16, then each count in the Timer8 object represents (2^16)*(174/20_000_000) = 0.5701632 seconds.
 * The total range for the Timer8 object is (2^8)*0.5701632 = 145.9617792 seconds.
 * The 10 second timeout value is 10/0.5701632 = 17.
 *
 * In this case a Timer8 object is sufficient if the resolution of 0.0712704 seconds is acceptable. If not, a Timer16 object
 * is required. The setting for TIMER_USEyy is determined by the highest resolution Timer object that the application
 * requires or uses.
 *
 * Timer32 properties TIMER_RUN32 defined):
 * - data area 9 bytes
 * - timer resolution 32bits
 * - shift parameter 0
 * - time unit (INTPERIOD/CPUFREQ) seconds
 * - roundtrip time (2^32)*timeunit
 *
 * Timer24 properties TIMER_RUNxx defined, xx = 24 or 32):
 * - data area 7 bytes
 * - timer resolution 24bits
 * - shift parameter 0 - (xx-24)
 * - time unit (2^shift)*(INTPERIOD/CPUFREQ) seconds
 * - roundtrip time (2^24)*timeunit but not exceeding (2^xx)*(INTPERIOD/CPUFREQ)
 *
 * Timer16 properties TIMER_RUNxx defined, xx = 16 or 24 or 32):
 * - data area 5 bytes
 * - timer resolution 16bits
 * - shift parameter 0 - (xx-16)
 * - time unit (2^shift)*(INTPERIOD/CPUFREQ) seconds
 * - roundtrip time (2^16)*timeunit but not exceeding (2^xx)*(INTPERIOD/CPUFREQ)
 *
 * Timer8 properties TIMER_RUNxx defined, xx = 08 or 16 or 24 or 32):
 * - data area 3 bytes
 * - timer resolution 8bits
 * - shift parameter 0 - (xx-8)
 * - time unit (2^shift)*(INTPERIOD/CPUFREQ) seconds
 * - roundtrip time (2^8)*timeunit but not exceeding (2^xx)*(INTPERIOD/CPUFREQ)
 *
 * How to use:
 * \code
 * Timer16 t; //declare a 16bits timer
 * Timer_mark(t,timeoutvalue,shiftvalue); //mark the timer
 * while (!Timer_timeout(t)) {
 *   \\execute code while timer has not expired
 * }
 * Timer_mark(t,timeoutvalue,shiftvalue);
 * while (1) {
 *   if (Timer_timeout(t)) {
 *     Timer_mark(t,timeoutvalue,shiftvalue);
 *     //execute some code 'on time'
 *   }
 * }
 * \endcode
 *
 * Timer objects can be used in Tasks, if declared globally or static locally, so the Timer object survives taskswitches.
 * \code
 * TASK myTask(void)
 * {
 *   static Timer16 t;
 *   //task code
 * }
 * \endcode
 */

#define INTRATE ((2*CPUFREQ/(INTPERIOD))+1)/2

/**
 * \brief Calculate timer units for specified number of microseconds
 *
 * The macro TIMER_USEC() calculates the number of timer units for specified number of microseconds.
 * The parameters must be compile time constant values.
 *
 * \param usec Time in microsecond units.
 * \param shift The number of bits the free running timer count is rightshifted.
 */
#define TIMER_USEC(usec,shift) \
\
((2*(usec)*(INTRATE)/1000000/(1<<shift))+1)/2

/**
 * \brief Calculate timer units for specified number of milliseconds
 *
 * The macro TIMER_MSEC() calculates the number of timer units for specified number of milliseconds.
 * The parameters must be compile time constant values.
 *
 * \param msec Time in millisecond units.
 * \param shift The number of bits the free running timer count is rightshifted.
 */
#define TIMER_MSEC(msec,shift) \
\
((2*(msec)*(INTRATE)/1000/(1<<shift))+1)/2

/**
 * \brief Calculate timer units for specified number of seconds
 *
 * The macro TIMER_SEC() calculates the number of timer units for specified number of seconds.
 * The parameters must be compile time constant values.
 *
 * \param sec Time in second units.
 * \param shift The number of bits the free running timer count is rightshifted.
 */
#define TIMER_SEC(sec,shift) \
\
((2*(sec)*(INTRATE)/(1<<shift))+1)/2

#if defined TIMER_RUN32
#define TIMER_RUN24
#define TIMER_RUN16

#if defined TIMER_USE32
#define TIMER32
#define TIMER24
#define TIMER16
#elif defined TIMER_USE24
#define TIMER24
#define TIMER16
#elif defined TIMER_USE16
#define TIMER16
#endif

#elif defined TIMER_RUN24
#define TIMER_RUN16

#if defined TIMER_USE24
#define TIMER24
#define TIMER16
#elif defined TIMER_USE16
#define TIMER16
#endif

#elif defined TIMER_RUN16

#if defined TIMER_USE16
#define TIMER16
#endif

#endif

#define TIMER_SHIFT  0
#define TIMER_START0 1
#define TIMER_STOP0  2
/**
 * \brief Timer8 object
 *
 * This object provides a timer with 8bit resolution. It is always available and requires
 * the least memory. This object occupies 3 ram bytes.
 */
typedef struct {
  char shift;
  char start0;
  char stop0;
} Timer8;

#ifdef TIMER16
#define TIMER_START1 3
#define TIMER_STOP1  4
/**
 * \brief Timer16 object
 *
 * This object provides a timer with 16bit resolution. It is only available when
 * at least TIMER_RUN16 and TIMER_USE16 are defined. This object occupies
 * 5 ram bytes.
 */
typedef struct {
  char shift;
  char start0;
  char stop0;
  char start1;
  char stop1;
} Timer16;
#endif

#ifdef TIMER24
#define TIMER_START2 5
#define TIMER_STOP2  6
/**
 * \brief Timer24 object
 *
 * This object provides a timer with 24bit resolution. It is only available when
 * at least TIMER_RUN24 and TIMER_USE24 are defined. This object occupies
 * 7 ram bytes.
 */
typedef struct {
  char shift;
  char start0;
  char stop0;
  char start1;
  char stop1;
  char start2;
  char stop2;
} Timer24;
#endif

#ifdef TIMER32
#define TIMER_START3 7
#define TIMER_STOP3  8
/**
 * \brief Timer32 object
 *
 * This object provides a timer with 32bit resolution. It is only available when
 * TIMER_RUN32 and TIMER_USE32 are defined. This object occupies
 * 9 ram bytes.
 */
typedef struct {
  char shift;
  char start0;
  char stop0;
  char start1;
  char stop1;
  char start2;
  char stop2;
  char start3;
  char stop3;
} Timer32;
#endif

#if defined TIMER_RUN32
static char TimerCount[4] @ TIMER_DATA+0; //32bit counter
static char TimerLatched[4] @ TIMER_DATA+4; //32bit latch
static char *TimerTimer @ TIMER_DATA+8; //placeholder for pointer to Timer object
static char TimerFlags @ TIMER_DATA+9;
#elif defined TIMER_RUN24
static char TimerCount[3] @ TIMER_DATA+0; //24bit counter
static char TimerLatched[3] @ TIMER_DATA+3; //24bit latch
static char *TimerTimer @ TIMER_DATA+6; //placeholder for pointer to Timer object
static char TimerFlags @ TIMER_DATA+7;
#elif defined TIMER_RUN16
static char TimerCount[2] @ TIMER_DATA+0; //16bit counter
static char TimerLatched[2] @ TIMER_DATA+2; //16bit latch
static char *TimerTimer @ TIMER_DATA+4; //placeholder for pointer to Timer object
static char TimerFlags @ TIMER_DATA+5;
#else
static char TimerCount[1] @ TIMER_DATA+0; //8bit counter
static char TimerLatched[1] @ TIMER_DATA+1; //8bit latch
static char *TimerTimer @ TIMER_DATA+2; //placeholder for pointer to Timer object
static char TimerFlags @ TIMER_DATA+3;
#endif
static char *TimerPassed @ TimerLatched[0]; //placeholder for pointer to variable that will hold passed ticks
static bit TimerLatch @ TimerFlags.0; //set by mainline code to indicate latching
static bit TimerTO1 @ TimerFlags.1; //used by timeout calculation
static bit TimerTO2 @ TimerFlags.2; //used by timeout calculation
static bit TimerTask @ TimerFlags.3; //to save TaskEnable status
static bit TimerTO3 @ TimerFlags.4; //set if start < stop (timer.shift bit 5)
static bit TimerT16 @ TimerFlags.5; //set if Timer16 type
static bit TimerT24 @ TimerFlags.6; //set if Timer24 type
static bit TimerT32 @ TimerFlags.7; //set if Timer32 type

/**
 * \brief Interrupt code for free running timer
 *
 * The macro Timer_ISR is the interrupt part of the free running timer. It is here that the free running
 * timer count is incremented every interrupt cycle.
 */
#define Timer_ISR \
{ \
#pragma update_FSR 0 \
  FSR = TIMER_DATA; \
  TimerCount[0]++; \
#ifdef TIMER_RUN16 \
  if (Zero_) { \
    TimerCount[1]++; \
#ifdef TIMER_RUN24 \
	if (Zero_) { \
      TimerCount[2]++; \
#ifdef TIMER_RUN32 \
	  if (Zero_) { \
	    TimerCount[3]++; \
	  } \
#endif \
	} \
#endif \
  } \
#endif \
  TimerLatch = 0; \
#pragma update_FSR 1 \
}

/**
 * \brief Interrupt code for free running timer
 *
 * Identical to the macro Timer_ISR.
 * The macro Timer_ISR is the interrupt part of the free running timer. It is here that the free running
 * timer count is incremented every interrupt cycle.
 */
void Timer_isr(void)
{
#pragma update_FSR 0
  FSR = TIMER_DATA;
  TimerCount[0]++;
#ifdef TIMER_RUN16
  if (Zero_) {
    TimerCount[1]++;
#ifdef TIMER_RUN24
	if (Zero_) {
      TimerCount[2]++;
#ifdef TIMER_RUN32
	  if (Zero_) {
	    TimerCount[3]++;
	  }
#endif
	}
#endif
  }
#endif
  TimerLatch = 0;
#pragma update_FSR 1
}

#ifdef _TASK_H_

/**
 * \brief Task Disable
 *
 * INTERNAL FUNCTION.\n
 * The function Timer_TaskDisable() saves the current Task state and then disables the tasks.\n
 * This is to prevent a task to overwrite internally used variables when mainline code accesses\n
 * Timer functions. Note that when a task accesses Timer functions, this is no problem\n
 * because a task cannot be interrupted by mainline code.
 * \n
 */
static void _Timer_TaskDisable(void)
{
  Carry = TaskEnabled;
  TimerTask = Carry;
  TaskEnabled = 0;
}

/**
 * \brief Task restore
 *
 * INTERNAL FUNCTION.\n
 * The function _Timer_TaskRestore() restores the current Task state\n
 * after completion of Timer functions.\n
 * \n
 */
static void _Timer_TaskRestore(void)
{
  Carry = TimerTask;
  TaskEnabled = Carry;
}

#endif

/**
 * \brief Latch free running timer
 *
 * INTERNAL FUNCTION.\n
 * The function Timer_latch() latches the current free running timer count.\n
 * This allows the value to be used even if the interrupt routine updates the timer.\n
 * \n
 */
static void _Timer_latch(void)
{
latch:
  TimerLatch = 1; //check for interrupt
  TimerLatched[0] = TimerCount[0];
#ifdef TIMER_RUN16
  TimerLatched[1] = TimerCount[1];
#endif
#ifdef TIMER_RUN24
  TimerLatched[2] = TimerCount[2];
#endif
#ifdef TIMER_RUN32
  TimerLatched[3] = TimerCount[3];
#endif
  if (!TimerLatch) goto latch; //interrupted, so latch again
  TimerLatch = 0;
  while (_MainTemp > 0) {
    Carry = 0;
#ifdef TIMER_RUN32
    TimerLatched[3] = rr(TimerLatched[3]);
#endif
#ifdef TIMER_RUN24
    TimerLatched[2] = rr(TimerLatched[2]);
#endif
#ifdef TIMER_RUN16
    TimerLatched[1] = rr(TimerLatched[1]);
#endif
    TimerLatched[0] = rr(TimerLatched[0]);
	_MainTemp--;
  }
}

/**
 * \brief Move Timer object type parameters into variable TimerFlags.\n
 * Move Timer object shiftvalue into _MainTemp.\n
 *
 * INTERNAL FUNCTION.\n
 * \n
 */
static void _TimerSetFlags(void)
{
  TimerFlags &= 0x0F;
  W = TimerTimer[TIMER_SHIFT];
  _MainTemp = W;
  TimerTO3 = _MainTemp.5;
  W &= 0xC0;
  switch (W) {
    case 0x40: W = 0x20;
	           break;
	case 0x80: W = 0x60;
	           break;
	case 0xC0: W = 0xE0;
	           break;
	default: W = 0;
  }
  TimerFlags |= W;
  _MainTemp &= 0x1F;
}

/**
 * \brief Write _MainTemp to char in array pointed to by variable TimerTimer
 *
 * INTERNAL FUNCTION.\n
 *
 * \n
 * \param W Index into array pointed to by variable TimerTimer.
 * \n
 */
static void _TimerSetField(char W)
{
  TimerTimer[W] = _MainTemp;
}

/**
 * \brief Add _MainTemp to char in array pointed to by variable TimerTimer
 *
 * INTERNAL FUNCTION.\n
 *
 * \n
 * \param W Index into array pointed to by variable TimerTimer.
 * \n
 */
static void _TimerAddField(char W)
{
  TimerTimer[W] += _MainTemp;
}

/**
 * \brief Increment char in array pointed to by variable TimerTimer
 *
 * INTERNAL FUNCTION.\n
 *
 * \n
 * \param W Index into array pointed to by variable TimerTimer.
 * \n
 */
static void _TimerIncField(char W)
{
  TimerTimer[W]++;
}

/**
 * \brief Load _MainTemp with char in array pointed to by variable TimerTimer
 *
 * INTERNAL FUNCTION.\n
 *
 * \n
 * \param W Index into array pointed to by variable TimerTimer.
 * \n
 */
static void _TimerGetField(char W)
{
  _MainTemp = TimerTimer[W];
}

/**
 * \brief Load _MainTemp with char in array TimerLatched
 *
 * INTERNAL FUNCTION.\n
 *
 * \n
 * \param W Index into array TimerLatched.
 * \n
 */
static void _TimerGetLatched(char W)
{
  _MainTemp = W;
  W = TimerLatched[_MainTemp];
  _MainTemp = W;
}

/**
 * \brief Write inverted _MainTemp to char in array pointed to by variable TimerPassed
 *
 * INTERNAL FUNCTION.\n
 *
 * \n
 * \param W Index into array pointed to by variable TimerPassed.
 * \n
 */
static void _TimerSetPassed(char W)
{
  TimerPassed[W] = ~_MainTemp;
}

/**
 * \brief Get current free running timer value
 *
 * The function Timer_timer() returns the current free running timer value. This is an unsigned value
 * that gets incremented every interrupt cycle.
 *
 * How to use:
 * \code
 * long ticks;
 * Timer_timer(ticks); //get current count from free running 16bits timer into variable ticks
 * \endcode
 *
 * \param count Variable that will hold the current free running timer count. The variable size must be
 * identical to the TIMER_RUNxx used. RUN08 requires char, RUN16 requires long, RUN24 requires
 * uns24, RUN32 requires uns32.
 */
#define Timer_timer(count) \
{ \
#ifdef _TASK_H_ \
  _Timer_TaskDisable(); \
#endif \
  TimerTimer = &(count); \
  _Timer_timer(); \
}

/**
 * \brief Get current free running timer value
 *
 * INTERNAL FUNCTION.\n
 * The function _Timer_timer() is only to be called from macro Timer_timer().\n
 * \n
 */
static void _Timer_timer()
{
  _MainTemp = 0;
  _Timer_latch();
  _TimerGetLatched(0);
  _TimerSetField(0);
#ifdef TIMER_RUN16
  _TimerGetLatched(1);
  _TimerSetField(1);
#endif
#ifdef TIMER_RUN24
  _TimerGetLatched(2);
  _TimerSetField(2);
#endif
#ifdef TIMER_RUN32
  _TimerGetLatched(3);
  _TimerSetField(3);
#endif
#ifdef _TASK_H_
  _Timer_TaskRestore();
#endif
}

/**
 * \brief Get the number of passed ticks since the last call to Timer_mark()
 *
 * The function Timer_passedticks() returns the number of passed ticks since the last
 * call to Timer_mark().
 *
 * \param timer Timer object that has been marked.
 * \param passed Variable that will hold the passed time in ticks. The variable size must be identical to the Timer object type.
 * Timer8 requires char, Timer16 requires long, Timer24 requires uns24, Timer32 requires uns32.
 */
#define Timer_passedticks(timer,passed) \
{ \
#ifdef _TASK_H_ \
  _Timer_TaskDisable(); \
#endif \
  TimerPassed = &(passed); \
  TimerTimer = &(timer); \
  _Timer_passedticks(); \
}

/**
 * \brief Get the number of passed ticks since the last call to Timer_mark()
 *
 * INTERNAL FUNCTION.\n
 * The function _Timer_passedticks() must only be called from the macro Timer_passedticks().\n
 * \n
 */
static void _Timer_passedticks(void)
{
  _TimerSetFlags(); //move shift into flags, _MainTemp = shift
  _TimerGetField(TIMER_START0); //get start
  _TimerSetPassed(0); //and move inverted start into passed
#ifdef TIMER16
  if (TimerT16) {
    _TimerGetField(TIMER_START1);
    _TimerSetPassed(1);
  }
#ifdef TIMER24
  if (TimerT24) {
    _TimerGetField(TIMER_START2);
    _TimerSetPassed(2);
  }
#ifdef TIMER32
  if (TimerT32) {
    _TimerGetField(TIMER_START3);
    _TimerSetPassed(3);
  }
#endif
#endif
#endif
  TimerTimer = TimerPassed; //pointer to passed
  _TimerIncField(0); //increment inverted start value to make it -start
#ifdef TIMER16
  if (TimerT16) {
    if (Zero_) { //ripple carry
      _TimerIncField(1);
#ifdef TIMER24
	  if (TimerT24) {
	    if (Zero_) {
	      _TimerIncField(2);
#ifdef TIMER32
		  if (TimerT32) {
	        if (Zero_) {
	          _TimerIncField(3);
	        }
		  }
#endif
	    }
	  }
#endif
    }
  }
#endif
  _Timer_latch();
  _TimerGetLatched(0); //get latched
  _TimerAddField(0); //add to passed
#ifdef TIMER16
  if (TimerT16) {
    if (Carry) { //ripple carry
      _TimerIncField(1);
#ifdef TIMER24
	  if (TimerT24) {
	    if (Zero_) {
	      _TimerIncField(2);
#ifdef TIMER32
		  if (TimerT32) {
	        if (Zero_) {
	          _TimerIncField(3);
	        }
		  }
#endif
	    }
	  }
#endif
    }
    _TimerGetLatched(1);
    _TimerAddField(1);
  }
#endif
#ifdef TIMER24
  if (TimerT24) {
    if (Carry) {
      _TimerIncField(2);
#ifdef TIMER32
	  if (TimerT32) {
	    if (Zero_) {
	      _TimerIncField(3);
	    }
	  }
#endif
    }
    _TimerGetLatched(2);
    _TimerAddField(2);
  }
#endif
#ifdef TIMER32
  if (TimerT32) {
    if (Carry) {
      _TimerIncField(3);
    }
    _TimerGetLatched(3);
    _TimerAddField(3);
  }
#endif
#ifdef _TASK_H_
  _Timer_TaskRestore();
#endif
}

/**
 * \brief Mark timer
 *
 * The function Timer_mark() marks a Timer object with an expiration value. This value is the number
 * of ticks that the free running timer (after rightshift) must advance before the Timer object expires.
 *
 * How to use:
 * \code
 * Timer16 t;
 * Timer_mark(t,4000,6); //let t expire in 4000 ticks, ignore lower 6 bits of free timer count
 * while (!Timer_timeout(t)) ; //delay for 4000 ticks
 * \endcode
 *
 * \param timer Timer16 object that will hold the timer expiration value.
 * \param timeout Number of ticks that the free running timer must advance before timer expires.
 * \param shift The number of bits the free running timer count is rightshifted.
 */
#define Timer_mark(timer,timeout,shift) \
{ \
#ifdef _TASK_H_ \
  _Timer_TaskDisable(); \
#endif \
  TimerTimer = &(timer); \
  _MainTemp = (shift) & 0x1F; \
#if sizeof(timer) == 9 \
  _MainTemp |= 0xC0; \
#elif sizeof(timer) == 7 \
  _MainTemp |= 0x80; \
#elif sizeof(timer) == 5 \
  _MainTemp |= 0x40; \
#endif \
  TimerTimer[TIMER_SHIFT] = _MainTemp; \
#if sizeof(timer) == 9 \
  _MainTemp = ((timeout) >> 24) & 0xFF; \
  _TimerSetField(TIMER_STOP3); \
#endif \
#if sizeof(timer) >= 7 \
  _MainTemp = ((timeout) >> 16) & 0xFF; \
  _TimerSetField(TIMER_STOP2); \
#endif \
#if sizeof(timer) >= 5 \
  _MainTemp = ((timeout) >> 8) & 0xFF; \
  _TimerSetField(TIMER_STOP1); \
#endif \
  W = (timeout) & 0xFF; \
  _Timer_mark(); \
}

/**
 * \brief Mark timer
 *
 * INTERNAL FUNCTION.\n
 * This function is only to be called from the macro Timer_mark();\n
 * \n
 */
static void _Timer_mark(void)
{
  _MainTemp = W;
  _TimerSetField(TIMER_STOP0);
  _TimerSetFlags();
  _Timer_latch();
  _TimerGetLatched(0);
  _TimerSetField(TIMER_START0);
  _TimerAddField(TIMER_STOP0);
#ifdef TIMER16
  if (TimerT16) {
    if (Carry) {
      _TimerIncField(TIMER_STOP1);
#ifdef TIMER24
	  if (TimerT24) {
	    if (Zero_) {
	      _TimerIncField(TIMER_STOP2);
#ifdef TIMER32
		  if (TimerT32) {
	        if (Zero_) {
	          _TimerIncField(TIMER_STOP3);
	        }
		  }
#endif
	    }
	  }
#endif
    }
    _TimerGetLatched(1);
    _TimerSetField(TIMER_START1);
    _TimerAddField(TIMER_STOP1);
  }
#endif
#ifdef TIMER24
  if (TimerT24) {
    if (Carry) {
      _TimerIncField(TIMER_STOP2);
#ifdef TIMER32
	  if (TimerT32) {
	    if (Zero_) {
	      _TimerIncField(TIMER_STOP3);
	    }
	  }
#endif
    }
    _TimerGetLatched(2);
    _TimerSetField(TIMER_START2);
    _TimerAddField(TIMER_STOP2);
  }
#endif
#ifdef TIMER32
  if (TimerT32) {
    if (Carry) {
      _TimerIncField(TIMER_STOP3);
    }
    _TimerGetLatched(3);
    _TimerSetField(TIMER_START3);
    _TimerAddField(TIMER_STOP3);
  }
#endif
  TimerTimer[TIMER_SHIFT].5 = !Carry; // start < stop
#ifdef _TASK_H_
  _Timer_TaskRestore();
#endif
}

/**
 * \brief Mark timer for specific microseconds
 *
 * The function Timer_markUSEC() marks a Timer object with an expiration value. This value is the number
 * of microseconds that must pass before the Timer object expires.
 *
 * How to use:
 * \code
 * Timer16 t;
 * Timer_markUSEC(t,4000); //let t expire in 4000 microseconds
 * while (!Timer_timeout(t)) ; //delay for 4000 microseconds
 * \endcode
 *
 * \param timer Timer object that will hold the timer expiration value.
 * \param timeout Number of microseconds that must pass before timer expires.
 */
#define Timer_markUSEC(timer,timeout) \
{ \
#if sizeof(timer) == 9 \
  Timer_mark(timer,TIMER_USEC(timeout,0),0) \
#elif sizeof(timer) == 7 \
  #if defined TIMER32 \
    #if TIMER_USEC(timeout,0) < 16777216 \
      Timer_mark(timer,TIMER_USEC(timeout,0),0) \
    #elif TIMER_USEC(timeout,1) < 16777216 \
      Timer_mark(timer,TIMER_USEC(timeout,1),1) \
    #elif TIMER_USEC(timeout,2) < 16777216 \
      Timer_mark(timer,TIMER_USEC(timeout,2),2) \
    #elif TIMER_USEC(timeout,3) < 16777216 \
      Timer_mark(timer,TIMER_USEC(timeout,3),3) \
    #elif TIMER_USEC(timeout,4) < 16777216 \
      Timer_mark(timer,TIMER_USEC(timeout,4),4) \
    #elif TIMER_USEC(timeout,5) < 16777216 \
      Timer_mark(timer,TIMER_USEC(timeout,5),5) \
    #elif TIMER_USEC(timeout,6) < 16777216 \
      Timer_mark(timer,TIMER_USEC(timeout,6),6) \
    #elif TIMER_USEC(timeout,7) < 16777216 \
      Timer_mark(timer,TIMER_USEC(timeout,7),7) \
    #elif TIMER_USEC(timeout,8) < 16777216 \
      Timer_mark(timer,TIMER_USEC(timeout,8),8) \
    #else \
      #error Timeout cannot be realised with timer object size \
    #endif \
  #else \
    #if TIMER_USEC(timeout,0) < 16777216 \
      Timer_mark(timer,TIMER_USEC(timeout,0),0) \
	#else \
      #error Timeout cannot be realised with free running timer size \
	#endif \
  #endif \
#elif sizeof(timer) == 5 \
  #if defined TIMER32 \
    #if TIMER_USEC(timeout,0) < 65536 \
      Timer_mark(timer,TIMER_USEC(timeout,0),0) \
    #elif TIMER_USEC(timeout,1) < 65536 \
      Timer_mark(timer,TIMER_USEC(timeout,1),1) \
    #elif TIMER_USEC(timeout,2) < 65536 \
      Timer_mark(timer,TIMER_USEC(timeout,2),2) \
    #elif TIMER_USEC(timeout,3) < 65536 \
      Timer_mark(timer,TIMER_USEC(timeout,3),3) \
    #elif TIMER_USEC(timeout,4) < 65536 \
      Timer_mark(timer,TIMER_USEC(timeout,4),4) \
    #elif TIMER_USEC(timeout,5) < 65536 \
      Timer_mark(timer,TIMER_USEC(timeout,5),5) \
    #elif TIMER_USEC(timeout,6) < 65536 \
      Timer_mark(timer,TIMER_USEC(timeout,6),6) \
    #elif TIMER_USEC(timeout,7) < 65536 \
      Timer_mark(timer,TIMER_USEC(timeout,7),7) \
    #elif TIMER_USEC(timeout,8) < 65536 \
      Timer_mark(timer,TIMER_USEC(timeout,8),8) \
    #elif TIMER_USEC(timeout,9) < 65536 \
      Timer_mark(timer,TIMER_USEC(timeout,9),9) \
    #elif TIMER_USEC(timeout,10) < 65536 \
      Timer_mark(timer,TIMER_USEC(timeout,10),10) \
    #elif TIMER_USEC(timeout,11) < 65536 \
      Timer_mark(timer,TIMER_USEC(timeout,11),11) \
    #elif TIMER_USEC(timeout,12) < 65536 \
      Timer_mark(timer,TIMER_USEC(timeout,12),12) \
    #elif TIMER_USEC(timeout,13) < 65536 \
      Timer_mark(timer,TIMER_USEC(timeout,13),13) \
    #elif TIMER_USEC(timeout,14) < 65536 \
      Timer_mark(timer,TIMER_USEC(timeout,14),14) \
    #elif TIMER_USEC(timeout,15) < 65536 \
      Timer_mark(timer,TIMER_USEC(timeout,15),15) \
    #elif TIMER_USEC(timeout,16) < 65536 \
      Timer_mark(timer,TIMER_USEC(timeout,16),16) \
    #else \
      #error Timeout cannot be realised with timer object \
    #endif \
  #elif defined TIMER24 \
    #if TIMER_USEC(timeout,0) < 65536 \
      Timer_mark(timer,TIMER_USEC(timeout,0),0) \
    #elif TIMER_USEC(timeout,1) < 65536 \
      Timer_mark(timer,TIMER_USEC(timeout,1),1) \
    #elif TIMER_USEC(timeout,2) < 65536 \
      Timer_mark(timer,TIMER_USEC(timeout,2),2) \
    #elif TIMER_USEC(timeout,3) < 65536 \
      Timer_mark(timer,TIMER_USEC(timeout,3),3) \
    #elif TIMER_USEC(timeout,4) < 65536 \
      Timer_mark(timer,TIMER_USEC(timeout,4),4) \
    #elif TIMER_USEC(timeout,5) < 65536 \
      Timer_mark(timer,TIMER_USEC(timeout,5),5) \
    #elif TIMER_USEC(timeout,6) < 65536 \
      Timer_mark(timer,TIMER_USEC(timeout,6),6) \
    #elif TIMER_USEC(timeout,7) < 65536 \
      Timer_mark(timer,TIMER_USEC(timeout,7),7) \
    #elif TIMER_USEC(timeout,8) < 65536 \
      Timer_mark(timer,TIMER_USEC(timeout,8),8) \
    #else \
      #error Timeout cannot be realised with timer object \
    #endif \
  #else \
    #if TIMER_USEC(timeout,0) < 65536 \
      Timer_mark(timer,TIMER_USEC(timeout,0),0) \
	#else \
      #error Timeout cannot be realised with free running timer size \
	#endif \
  #endif \
#else \
  #if defined TIMER32 \
    #if TIMER_USEC(timeout,0) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,0),0) \
    #elif TIMER_USEC(timeout,1) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,1),1) \
    #elif TIMER_USEC(timeout,2) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,2),2) \
    #elif TIMER_USEC(timeout,3) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,3),3) \
    #elif TIMER_USEC(timeout,4) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,4),4) \
    #elif TIMER_USEC(timeout,5) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,5),5) \
    #elif TIMER_USEC(timeout,6) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,6),6) \
    #elif TIMER_USEC(timeout,7) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,7),7) \
    #elif TIMER_USEC(timeout,8) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,8),8) \
    #elif TIMER_USEC(timeout,9) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,9),9) \
    #elif TIMER_USEC(timeout,10) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,10),10) \
    #elif TIMER_USEC(timeout,11) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,11),11) \
    #elif TIMER_USEC(timeout,12) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,12),12) \
    #elif TIMER_USEC(timeout,13) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,13),13) \
    #elif TIMER_USEC(timeout,14) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,14),14) \
    #elif TIMER_USEC(timeout,15) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,15),15) \
    #elif TIMER_USEC(timeout,16) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,16),16) \
    #elif TIMER_USEC(timeout,17) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,17),17) \
    #elif TIMER_USEC(timeout,18) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,18),18) \
    #elif TIMER_USEC(timeout,19) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,19),19) \
    #elif TIMER_USEC(timeout,20) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,20),20) \
    #elif TIMER_USEC(timeout,21) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,21),21) \
    #elif TIMER_USEC(timeout,22) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,22),22) \
    #elif TIMER_USEC(timeout,23) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,23),23) \
    #elif TIMER_USEC(timeout,24) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,24),24) \
    #else \
      #error Timeout cannot be realised with timer object \
    #endif \
  #elif defined TIMER24 \
    #if TIMER_USEC(timeout,0) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,0),0) \
    #elif TIMER_USEC(timeout,1) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,1),1) \
    #elif TIMER_USEC(timeout,2) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,2),2) \
    #elif TIMER_USEC(timeout,3) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,3),3) \
    #elif TIMER_USEC(timeout,4) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,4),4) \
    #elif TIMER_USEC(timeout,5) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,5),5) \
    #elif TIMER_USEC(timeout,6) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,6),6) \
    #elif TIMER_USEC(timeout,7) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,7),7) \
    #elif TIMER_USEC(timeout,8) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,8),8) \
    #elif TIMER_USEC(timeout,9) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,9),9) \
    #elif TIMER_USEC(timeout,10) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,10),10) \
    #elif TIMER_USEC(timeout,11) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,11),11) \
    #elif TIMER_USEC(timeout,12) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,12),12) \
    #elif TIMER_USEC(timeout,13) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,13),13) \
    #elif TIMER_USEC(timeout,14) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,14),14) \
    #elif TIMER_USEC(timeout,15) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,15),15) \
    #elif TIMER_USEC(timeout,16) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,16),16) \
    #else \
      #error Timeout cannot be realised with timer object \
    #endif \
  #elif defined TIMER16 \
    #if TIMER_USEC(timeout,0) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,0),0) \
    #elif TIMER_USEC(timeout,1) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,1),1) \
    #elif TIMER_USEC(timeout,2) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,2),2) \
    #elif TIMER_USEC(timeout,3) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,3),3) \
    #elif TIMER_USEC(timeout,4) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,4),4) \
    #elif TIMER_USEC(timeout,5) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,5),5) \
    #elif TIMER_USEC(timeout,6) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,6),6) \
    #elif TIMER_USEC(timeout,7) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,7),7) \
    #elif TIMER_USEC(timeout,8) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,8),8) \
    #else \
      #error Timeout cannot be realised with timer object \
    #endif \
  #else \
    #if TIMER_USEC(timeout,0) < 256 \
      Timer_mark(timer,TIMER_USEC(timeout,0),0) \
	#else \
      #error Timeout cannot be realised with free running timer size \
	#endif \
  #endif \
#endif \
}

/**
 * \brief Mark timer for specific milliseconds
 *
 * The function Timer_markMSEC() marks a Timer object with an expiration value. This value is the number
 * of milliseconds that must pass before the Timer object expires.
 *
 * How to use:
 * \code
 * Timer16 t;
 * Timer_markMSEC(t,4000); //let t expire in 4000 milliseconds
 * while (!Timer_timeout(t)) ; //delay for 4000 milliseconds
 * \endcode
 *
 * \param timer Timer object that will hold the timer expiration value.
 * \param timeout Number of milliseconds that must pass before timer expires.
 */
#define Timer_markMSEC(timer,timeout) \
{ \
#if sizeof(timer) == 9 \
  Timer_mark(timer,TIMER_MSEC(timeout,0),0) \
#elif sizeof(timer) == 7 \
  #if defined TIMER32 \
    #if TIMER_MSEC(timeout,0) < 16777216 \
      Timer_mark(timer,TIMER_MSEC(timeout,0),0) \
    #elif TIMER_MSEC(timeout,1) < 16777216 \
      Timer_mark(timer,TIMER_MSEC(timeout,1),1) \
    #elif TIMER_MSEC(timeout,2) < 16777216 \
      Timer_mark(timer,TIMER_MSEC(timeout,2),2) \
    #elif TIMER_MSEC(timeout,3) < 16777216 \
      Timer_mark(timer,TIMER_MSEC(timeout,3),3) \
    #elif TIMER_MSEC(timeout,4) < 16777216 \
      Timer_mark(timer,TIMER_MSEC(timeout,4),4) \
    #elif TIMER_MSEC(timeout,5) < 16777216 \
      Timer_mark(timer,TIMER_MSEC(timeout,5),5) \
    #elif TIMER_MSEC(timeout,6) < 16777216 \
      Timer_mark(timer,TIMER_MSEC(timeout,6),6) \
    #elif TIMER_MSEC(timeout,7) < 16777216 \
      Timer_mark(timer,TIMER_MSEC(timeout,7),7) \
    #elif TIMER_MSEC(timeout,8) < 16777216 \
      Timer_mark(timer,TIMER_MSEC(timeout,8),8) \
    #else \
      #error Timeout cannot be realised with timer object size \
    #endif \
  #else \
    #if TIMER_MSEC(timeout,0) < 16777216 \
      Timer_mark(timer,TIMER_MSEC(timeout,0),0) \
	#else \
      #error Timeout cannot be realised with free running timer size \
	#endif \
  #endif \
#elif sizeof(timer) == 5 \
  #if defined TIMER32 \
    #if TIMER_MSEC(timeout,0) < 65536 \
      Timer_mark(timer,TIMER_MSEC(timeout,0),0) \
    #elif TIMER_MSEC(timeout,1) < 65536 \
      Timer_mark(timer,TIMER_MSEC(timeout,1),1) \
    #elif TIMER_MSEC(timeout,2) < 65536 \
      Timer_mark(timer,TIMER_MSEC(timeout,2),2) \
    #elif TIMER_MSEC(timeout,3) < 65536 \
      Timer_mark(timer,TIMER_MSEC(timeout,3),3) \
    #elif TIMER_MSEC(timeout,4) < 65536 \
      Timer_mark(timer,TIMER_MSEC(timeout,4),4) \
    #elif TIMER_MSEC(timeout,5) < 65536 \
      Timer_mark(timer,TIMER_MSEC(timeout,5),5) \
    #elif TIMER_MSEC(timeout,6) < 65536 \
      Timer_mark(timer,TIMER_MSEC(timeout,6),6) \
    #elif TIMER_MSEC(timeout,7) < 65536 \
      Timer_mark(timer,TIMER_MSEC(timeout,7),7) \
    #elif TIMER_MSEC(timeout,8) < 65536 \
      Timer_mark(timer,TIMER_MSEC(timeout,8),8) \
    #elif TIMER_MSEC(timeout,9) < 65536 \
      Timer_mark(timer,TIMER_MSEC(timeout,9),9) \
    #elif TIMER_MSEC(timeout,10) < 65536 \
      Timer_mark(timer,TIMER_MSEC(timeout,10),10) \
    #elif TIMER_MSEC(timeout,11) < 65536 \
      Timer_mark(timer,TIMER_MSEC(timeout,11),11) \
    #elif TIMER_MSEC(timeout,12) < 65536 \
      Timer_mark(timer,TIMER_MSEC(timeout,12),12) \
    #elif TIMER_MSEC(timeout,13) < 65536 \
      Timer_mark(timer,TIMER_MSEC(timeout,13),13) \
    #elif TIMER_MSEC(timeout,14) < 65536 \
      Timer_mark(timer,TIMER_MSEC(timeout,14),14) \
    #elif TIMER_MSEC(timeout,15) < 65536 \
      Timer_mark(timer,TIMER_MSEC(timeout,15),15) \
    #elif TIMER_MSEC(timeout,16) < 65536 \
      Timer_mark(timer,TIMER_MSEC(timeout,16),16) \
    #else \
      #error Timeout cannot be realised with timer object \
    #endif \
  #elif defined TIMER24 \
    #if TIMER_MSEC(timeout,0) < 65536 \
      Timer_mark(timer,TIMER_MSEC(timeout,0),0) \
    #elif TIMER_MSEC(timeout,1) < 65536 \
      Timer_mark(timer,TIMER_MSEC(timeout,1),1) \
    #elif TIMER_MSEC(timeout,2) < 65536 \
      Timer_mark(timer,TIMER_MSEC(timeout,2),2) \
    #elif TIMER_MSEC(timeout,3) < 65536 \
      Timer_mark(timer,TIMER_MSEC(timeout,3),3) \
    #elif TIMER_MSEC(timeout,4) < 65536 \
      Timer_mark(timer,TIMER_MSEC(timeout,4),4) \
    #elif TIMER_MSEC(timeout,5) < 65536 \
      Timer_mark(timer,TIMER_MSEC(timeout,5),5) \
    #elif TIMER_MSEC(timeout,6) < 65536 \
      Timer_mark(timer,TIMER_MSEC(timeout,6),6) \
    #elif TIMER_MSEC(timeout,7) < 65536 \
      Timer_mark(timer,TIMER_MSEC(timeout,7),7) \
    #elif TIMER_MSEC(timeout,8) < 65536 \
      Timer_mark(timer,TIMER_MSEC(timeout,8),8) \
    #else \
      #error Timeout cannot be realised with timer object \
    #endif \
  #else \
    #if TIMER_MSEC(timeout,0) < 65536 \
      Timer_mark(timer,TIMER_MSEC(timeout,0),0) \
	#else \
      #error Timeout cannot be realised with free running timer size \
	#endif \
  #endif \
#else \
  #if defined TIMER32 \
    #if TIMER_MSEC(timeout,0) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,0),0) \
    #elif TIMER_MSEC(timeout,1) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,1),1) \
    #elif TIMER_MSEC(timeout,2) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,2),2) \
    #elif TIMER_MSEC(timeout,3) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,3),3) \
    #elif TIMER_MSEC(timeout,4) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,4),4) \
    #elif TIMER_MSEC(timeout,5) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,5),5) \
    #elif TIMER_MSEC(timeout,6) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,6),6) \
    #elif TIMER_MSEC(timeout,7) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,7),7) \
    #elif TIMER_MSEC(timeout,8) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,8),8) \
    #elif TIMER_MSEC(timeout,9) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,9),9) \
    #elif TIMER_MSEC(timeout,10) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,10),10) \
    #elif TIMER_MSEC(timeout,11) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,11),11) \
    #elif TIMER_MSEC(timeout,12) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,12),12) \
    #elif TIMER_MSEC(timeout,13) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,13),13) \
    #elif TIMER_MSEC(timeout,14) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,14),14) \
    #elif TIMER_MSEC(timeout,15) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,15),15) \
    #elif TIMER_MSEC(timeout,16) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,16),16) \
    #elif TIMER_MSEC(timeout,17) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,17),17) \
    #elif TIMER_MSEC(timeout,18) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,18),18) \
    #elif TIMER_MSEC(timeout,19) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,19),19) \
    #elif TIMER_MSEC(timeout,20) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,20),20) \
    #elif TIMER_MSEC(timeout,21) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,21),21) \
    #elif TIMER_MSEC(timeout,22) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,22),22) \
    #elif TIMER_MSEC(timeout,23) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,23),23) \
    #elif TIMER_MSEC(timeout,24) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,24),24) \
    #else \
      #error Timeout cannot be realised with timer object \
    #endif \
  #elif defined TIMER24 \
    #if TIMER_MSEC(timeout,0) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,0),0) \
    #elif TIMER_MSEC(timeout,1) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,1),1) \
    #elif TIMER_MSEC(timeout,2) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,2),2) \
    #elif TIMER_MSEC(timeout,3) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,3),3) \
    #elif TIMER_MSEC(timeout,4) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,4),4) \
    #elif TIMER_MSEC(timeout,5) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,5),5) \
    #elif TIMER_MSEC(timeout,6) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,6),6) \
    #elif TIMER_MSEC(timeout,7) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,7),7) \
    #elif TIMER_MSEC(timeout,8) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,8),8) \
    #elif TIMER_MSEC(timeout,9) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,9),9) \
    #elif TIMER_MSEC(timeout,10) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,10),10) \
    #elif TIMER_MSEC(timeout,11) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,11),11) \
    #elif TIMER_MSEC(timeout,12) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,12),12) \
    #elif TIMER_MSEC(timeout,13) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,13),13) \
    #elif TIMER_MSEC(timeout,14) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,14),14) \
    #elif TIMER_MSEC(timeout,15) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,15),15) \
    #elif TIMER_MSEC(timeout,16) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,16),16) \
    #else \
      #error Timeout cannot be realised with timer object \
    #endif \
  #elif defined TIMER16 \
    #if TIMER_MSEC(timeout,0) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,0),0) \
    #elif TIMER_MSEC(timeout,1) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,1),1) \
    #elif TIMER_MSEC(timeout,2) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,2),2) \
    #elif TIMER_MSEC(timeout,3) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,3),3) \
    #elif TIMER_MSEC(timeout,4) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,4),4) \
    #elif TIMER_MSEC(timeout,5) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,5),5) \
    #elif TIMER_MSEC(timeout,6) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,6),6) \
    #elif TIMER_MSEC(timeout,7) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,7),7) \
    #elif TIMER_MSEC(timeout,8) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,8),8) \
    #else \
      #error Timeout cannot be realised with timer object \
    #endif \
  #else \
    #if TIMER_MSEC(timeout,0) < 256 \
      Timer_mark(timer,TIMER_MSEC(timeout,0),0) \
	#else \
      #error Timeout cannot be realised with free running timer size \
	#endif \
  #endif \
#endif \
}

/**
 * \brief Get char from array pointed to by TimerTimer variable
 *
 * INTERNAL FUNCTION.\n
 * _MainTemp must hold the index into the array.\n
 *
 * \n
 * \return Indexed character.
 * \n
 */
static char _TimerIndexed(void)
{
  return TimerTimer[_MainTemp];
}

/**
 * \brief Calculate timer count difference
 *
 * INTERNAL FUNCTION.\n
 * The function _TimerDifference() calculates the difference between two timer counts.\n
 * The result is either negative (Carry clear) or positive/zero (Carry set).\n
 * It is called by the function _Timer_timeout().\n
 * \n
 */
static void _TimerDifference(void)
{
#ifdef TIMER32
  if (!TimerT32) goto T24;
  W = _TimerIndexed();
  W = TimerLatched[3] - W;
  if (Zero_) {
T24:
    _MainTemp--;
    _MainTemp--;
#endif
#ifdef TIMER24
    if (!TimerT24) goto T16;
    W = _TimerIndexed();
    W = TimerLatched[2] - W;
	if (Zero_) {
T16:
      _MainTemp--;
	  _MainTemp--;
#endif
#ifdef TIMER16
      if (!TimerT16) goto T08;
      W = _TimerIndexed();
      W = TimerLatched[1] - W;
	  if (Zero_) {
T08:
        _MainTemp--;
	    _MainTemp--;
#endif
        W = _TimerIndexed();
        W = TimerLatched[0] - W;
#ifdef TIMER16
	  }
#endif
#ifdef TIMER24
	}
#endif
#ifdef TIMER32
  }
#endif
}

/**
 * \brief Test for timer timeout
 *
 * The function Timer_timeout() tests if a Timer object has expired. This function is usually called
 * in a loop to either create a delay or to test wether some response is in time.
 *
 * How to use:
 * \code
 * long receive(void) { //get timely response or return -1
 *   Timer16 t;
 *   Timer_mark(t,2000,0);
 *   while (!Timer_timeout(t)) {
 *     if (vpUartRx_byteAvailable(rx)) {
 *       char value = vpUartRx_receiveByte(rx);
 *       return value;
 *     }
 *   }
 *   return -1;
 * }
 * \endcode
 *
 * \param timer Timer object that has been marked with an expiration value.
 * \return True if Timer object has expired.
 */
#define Timer_timeout(timer) \
\
_Timer_timeout(&timer)

/**
 * \brief Test for timer timeout
 *
 * INTERNAL FUNCTION.\n
 * The function _Timer_timeout() tests if a Timer object has expired.\n
 * It should only be called from the macro Timer_timeout().\n
 *
 * \n
 * \param W Address of Timer object that has been marked with an expiration value.
 * \return True if Timer object has expired.
 * \n
 */
static bit _Timer_timeout(char W)
{
#ifdef _TASK_H_
  _Timer_TaskDisable();
#endif
  TimerTimer = W;
  _TimerSetFlags();
  _Timer_latch();
  //p1 = (current >= stop)
#if defined TIMER32
  _MainTemp = TIMER_STOP3;
#elif defined TIMER24
  _MainTemp = TIMER_STOP2;
#elif defined TIMER16
  _MainTemp = TIMER_STOP1;
#else
  _MainTemp = TIMER_STOP0;
#endif
  _TimerDifference();
  TimerTO1 = Carry;
  //p2 = (current < start)
#if defined TIMER32
  _MainTemp = TIMER_START3;
#elif defined TIMER24
  _MainTemp = TIMER_START2;
#elif defined TIMER16
  _MainTemp = TIMER_START1;
#else
  _MainTemp = TIMER_START0;
#endif
  _TimerDifference();
  TimerTO2 = !Carry;
  if (TimerTO3) { // if (start < stop)
    TimerTO1 |= TimerTO2;
  }
  else {
    TimerTO1 &= TimerTO2;
  }
  W = TimerTO1;
#ifdef _TASK_H_
  _Timer_TaskRestore();
#endif
  return W;
}

#pragma library 0
#endif
// End of file Timer.h
