// File vpAdc.h for CC1B
#ifndef _VPADC_H_
#define _VPADC_H_
//include other libraries here
#include <system/memory.h>
#pragma library 1

/**
 * \file vpAdc.h
 * \brief Library for virtual peripheral vpAdc
 *
 * This library provides a virtual peripheral Sigma-Delta Analog to Digital converter.
 * The ADC resolution can be specified from 1 to 15 bits.
 */

/**
 * \brief Define a vpAdc
 *
 * The macro vpAdc15() defines a 15bits sigma delta ADC. Its parameters are stored in rom.
 * You install this ADC by calling vph = vpInstall(name) and you remove the ADC by
 * calling vpUninstall(vph).
 *
 * How to use:
 * \code
 * vpAdc(AD1,PORTPIN_RA2,LOWINPUT_LOWINPUT,PORTPIN_RB5,LOWOUTPUT_LOWOUTPUT,12)
 * char ad1 = vpInstall(AD1);
 * \endcode
 *
 * The above defines an ADC with input pin RA.2 and output pin RB.5 for 12 bits resolution.
 * Note the absent ';' after the definition.
 *
 * \param name Name for this ADC. The name must be unique as it is used as a label for rom data.\n
 * \param inpin Input pin to read the ADC state.
 * \param inconfig Configuration for the input pin. This specifies the pin states after installation
 * and removal of the ADC. Its format is XX_YY in which XX defines the state after installation
 * and YY defines the state after removal of the ADC. Possible values for XX and YY are:
 * LEAVE, LOWOUTPUT, HIGHOUTPUT, LOWINPUT, HIGHINPUT. Because the inpin
 * is an input pin, only LOWINPUT and HIGHINPUT make sense.
 * \param outpin Output pin to maintain input pin treshold level.
 * \param outconfig Configuration for the output pin. This specifies the pin states after installation
 * and removal of the ADC. Its format is XX_YY in which XX defines the state after installation
 * and YY defines the state after removal of the ADC. Possible values for XX and YY are:
 * LEAVE, LOWOUTPUT, HIGHOUTPUT, LOWINPUT, HIGHINPUT. Because the outpin
 * is an output pin, only LOWOUTPUT and HIGHOUTPUT make sense.
 * \param resolution The number of used bits in the ADC value. Range is 1 to 15,
 */
#define vpAdc(name,inpin,inconfig,outpin,outconfig,resolution) \
#ifndef vpAdc_installed \
#define vpAdc_installed \
#endif \
#pragma cdata. ##name = VP_ADC,12 \
#pragma cdata[] = ((inpin)>>8)&0x0F,(inpin)&0xFF,((outpin)>>8)&0x0F,(outpin)&0xFF \
#pragma cdata[] = (-(1<<resolution))&0xFF,(-(1<<resolution))>>8,(-(1<<resolution))&0xFF,(-(1<<resolution))>>8 \
#pragma cdata[] = 0,0,0,0 \
#pragma cdata. ##name##_pinconfig \
#pragma cdata[] = (inpin)>>8,(inpin)&0xFF,inconfig \
#pragma cdata[] = (outpin)>>8,(outpin)&0xFF,outconfig \
#pragma cdata[] = 0

static shadowDef char vpAdc_portIn @ 0x10;
  static shadowDef bit vpAdc_retrieving @ vpAdc_portIn.7;
static shadowDef char vpAdc_pinIn  @ 0x11;
static shadowDef char vpAdc_portOut @ 0x12;
static shadowDef char vpAdc_pinOut @ 0x13;
static shadowDef unsigned long vpAdc_bits @ 0x14;
static shadowDef unsigned long vpAdc_count @ 0x16;
static shadowDef unsigned long vpAdc_accum @ 0x18;
static shadowDef unsigned long vpAdc_value @ 0x1A;

/**
 * \brief Interrupt code for virtual peripheral ADC
 *
 * This part calculates the ADC value. It allows an n-bit value to be calculated which corresponds
 * directly (within noise variation limits) with the voltage (0-5V) present at the respective vpAdc
 * port input pin. This routine is timing critical and must be placed before any variable-execution-rate
 * code(like the UART, for example).
 */
#define vpAdc_ISR \
{ \
#pragma update_FSR 0 \
  W = vpAdc_portIn & 0x0F; \
  FSR = W; \
  W = ~INDF; \
  _IsrTemp = W; \
  FSR = _IsrBank; \
  W = vpAdc_pinIn; \
  _IsrTemp &= W; \
  Carry = !Zero_; \
  _IsrTemp = ~vpAdc_pinOut; \
  W = vpAdc_portOut & 0x0F; \
  FSR = W; \
  W = INDF; \
  W &= _IsrTemp; \
  _IsrTemp = ~_IsrTemp; \
  if (Carry) W |= _IsrTemp; \
  INDF = W; \
  FSR = _IsrBank; \
  if (!Carry) { \
    vpAdc_accum++; \
  } \
  vpAdc_count++; \
  if (Zero_) { \
    if (!vpAdc_retrieving) { \
      vpAdc_value = vpAdc_accum; \
	} \
    vpAdc_accum = 0; \
    vpAdc_count = vpAdc_bits; \
  } \
#pragma update_FSR 1 \
}

/**
 * \brief Interrupt code for virtual peripheral ADC
 *
 * Identical to macro vpAdc_ISR.
 * This part calculates the ADC value. It allows an n-bit value to be calculated which corresponds
 * directly (within noise variation limits) with the voltage (0-5V) present at the respective vpAdc
 * port input pin. This routine is timing critical and must be placed before any variable-execution-rate
 * code(like the UART, for example).
 */
void vpAdc_isr(void)
{
#pragma update_FSR 0
  FSR = _IsrBank;
  W = vpAdc_portIn & 0x0F;
  FSR = W;
  W = ~INDF;
  _IsrTemp = W;
  FSR = _IsrBank;
  W = vpAdc_pinIn;
  _IsrTemp &= W;
  Carry = !Zero_;
  _IsrTemp = ~vpAdc_pinOut;
  W = vpAdc_portOut & 0x0F;
  FSR = W;
  W = INDF;
  W &= _IsrTemp;
  _IsrTemp = ~_IsrTemp;
  if (Carry) W |= _IsrTemp;
  INDF = W;
  FSR = _IsrBank;
  if (!Carry) {
    vpAdc_accum++;
  }
  vpAdc_count++;
  if (Zero_) {
    if (!vpAdc_retrieving) {
      vpAdc_value = vpAdc_accum;
	}
    vpAdc_accum = 0;
    vpAdc_count = vpAdc_bits;
  }
#pragma update_FSR 1
}

/**
 * \brief Read ADC value
 *
 * The function vpAdc_read() reads the current ADC value. The lowbyte is returned
 * by this function, wheras the highbyte is returned in _MainTemp.
 *
 * \param W vpAdc address.
 * \return Lowbyte of current ADC value. The highbyte is returned in _MainTemp.
 */
char vpAdc_read(char W)
{
#pragma update_FSR 0
  FSR = W;
  vpAdc_retrieving = 1;
  W = vpAdc_value.high8;
  _MainTemp = W;
  W = vpAdc_value.low8;
  vpAdc_retrieving = 0;
#pragma update_FSR 1
  return W;
}

#pragma library 0
#endif
// End of file vpAdc.h
