// File vpPwm.h for CC1B
#ifndef _VPPWM_H_
#define _VPPWM_H_
//include other libraries here
#include <system/memory.h>
#pragma library 1

/**
 * \file vpPwm.h
 * \brief Library for virtual peripheral vpPwm
 *
 * This library provides a virtual peripheral Pulse Width Modulator.
 * Most pwm applications require a fixed period time and the application changes the dutycycle.
 * For this reason there are seperate functions to set the low time and the high time. The
 * changes come only in effect after calling the update function.
 */

/**
 * \brief Calculate pwm units for specified number of microseconds
 *
 * The macro PWM_USEC() calculates the number of pwm units for specified number of microseconds.
 *
 * \param usec Time in microsecond units.
 */
#define PWM_USEC(usec) \
\
((2*(usec)*(CPUFREQ/(INTPERIOD))/1000000/(VPRATIO))+1)/2

/**
 * \brief Calculate pwm units for specified number of milliseconds
 *
 * The macro PWM_MSEC() calculates the number of pwm units for specified number of milliseconds.
 *
 * \param msec Time in millisecond units.
 */
#define PWM_MSEC(msec) \
\
((2*(msec)*(CPUFREQ/(INTPERIOD))/1000/(VPRATIO))+1)/2

/**
 * \brief Define a vpPwm
 *
 * The macro vpPwm() defines a PWM. Its parameters are stored in rom.
 * You install this PWM by calling vph = vpInstall(name) and you remove the PWM by
 * calling vpUninstall(vph).
 *
 * How to use:
 * \code
 * vpPwm(PW1,PORTPIN_RB5,LOWOUTPUT_LOWOUTPUT,0x1000,0x0800)
 * char ad1 = vpInstall(AD1);
 * \endcode
 *
 * The above defines a PWM with output pin RB.5, low = 0x1000 isr cycles, high = 0x0800 isr cycles.
 * Note the absent ';' after the definition. The PWM period equals low time + high time.
 *
 * \param name Name for this PWM. The name must be unique as it is used as a label for rom data.\n
 * \param outpin Output pin that is driven high and low as set by parameters.
 * \param outconfig Configuration for the output pin. This specifies the pin states after installation
 * and removal of the PWM. Its format is XX_YY in which XX defines the state after installation
 * and YY defines the state after removal of the PWM. Possible values for XX and YY are:
 * LEAVE, LOWOUTPUT, HIGHOUTPUT, LOWINPUT, HIGHINPUT. Because the outpin
 * is an output pin, only LOWOUTPUT and HIGHOUTPUT make sense.
 * \param low The number of isr cycles that make up the low time.
 * \param high The number of isr cycles that make up the high time.
 */
#define vpPwm(name,outpin,outconfig,low,high) \
#ifndef vpPwm_installed \
#define vpPwm_installed \
#endif \
#pragma cdata. ##name = VP_PWM,12 \
#pragma cdata[] = ((outpin)>>8)&0x0F,(outpin)&0xFF \
#pragma cdata[] = (-low)&0xFF,(-low)>>8 \
#pragma cdata[] = (-low)&0xFF,(-low)>>8,(-high)&0xFF,(-high)>>8 \
#pragma cdata[] = (-low)&0xFF,(-low)>>8,(-high)&0xFF,(-high)>>8 \
#pragma cdata. ##name##_pinconfig \
#pragma cdata[] = (outpin)>>8,(outpin)&0xFF,outconfig \
#pragma cdata[] = 0

static shadowDef char vpPwm_portOut @ 0x10;
  static shadowDef bit vpPwm_updated @ vpPwm_portOut.7;
  static shadowDef bit vpPwm_state @ vpPwm_portOut.6;
static shadowDef char vpPwm_pinOut @ 0x11;
static shadowDef unsigned long vpPwm_count @ 0x12;
static shadowDef unsigned long vpPwm_Low @ 0x14;
static shadowDef unsigned long vpPwm_High @ 0x16;
static shadowDef unsigned long vpPwm_newLow @ 0x18;
static shadowDef unsigned long vpPwm_newHigh @ 0x1A;

/**
 * \brief Interrupt code for virtual peripheral PWM
 *
 * This part determines when to make the output pin low or high.
 * This routine is timing critical and must be placed before any variable-execution-rate
 * code(like the UART, for example).
 */
#define vpPwm_ISR \
{ \
#pragma update_FSR 0 \
  vpPwm_count++; \
  if (Zero_) { \
    vpPwm_portOut ^= 0x40; \
	W = vpPwm_Low.low8; \
    if (vpPwm_state) W = vpPwm_High.low8; \
	vpPwm_count.low8 = W; \
	W = vpPwm_Low.high8; \
    if (vpPwm_state) W = vpPwm_High.high8; \
	vpPwm_count.high8 = W; \
    _IsrTemp = vpPwm_pinOut; \
    W = vpPwm_portOut & 0x0F; \
    FSR = W; \
	W = _IsrTemp; \
    INDF ^= W; \
  } \
  else { \
    if (vpPwm_updated) { \
      vpPwm_Low = vpPwm_newLow; \
	  vpPwm_High = vpPwm_newHigh; \
	  vpPwm_updated = 0; \
    } \
  } \
#pragma update_FSR 1 \
}

/**
 * \brief Interrupt code for virtual peripheral PWM
 *
 * Identical to macro vpPwm_ISR.
 * This part determines when to make the output pin low or high.
 * This routine is timing critical and must be placed before any variable-execution-rate
 * code(like the UART, for example).
 */
void vpPwm_isr(void)
{
#pragma update_FSR 0
  vpPwm_count++;
  if (Zero_) {
    vpPwm_portOut ^= 0x40;
	W = vpPwm_Low.low8;
    if (vpPwm_state) W = vpPwm_High.low8;
	vpPwm_count.low8 = W;
	W = vpPwm_Low.high8;
    if (vpPwm_state) W = vpPwm_High.high8;
	vpPwm_count.high8 = W;
    _IsrTemp = vpPwm_pinOut;
    W = vpPwm_portOut & 0x0F;
    FSR = W;
	W = _IsrTemp;
    INDF ^= W;
  }
  else {
    if (vpPwm_updated) {
      vpPwm_Low = vpPwm_newLow;
	  vpPwm_High = vpPwm_newHigh;
	  vpPwm_updated = 0;
    }
  }
#pragma update_FSR 1
}

/**
 * \brief Set PWM low time
 *
 * The function vpPwm_low() sets the low time for the PWM signal. The low time is measured
 * in isr cycles. The PWM period time equals low time + high time.
 *
 * \param pwm Address of vpPwm.
 * \param low Low time measured in isr cycles.
 */
#define vpPwm_low(pwm,low) \
{ \
  _MainTemp = pwm; \
  W = (low) & 0xFF; \
  vpPwm_lowL(W); \
  W = (low) >> 8; \
  vpPwm_lowH(W); \
}

/**
 * \brief Setup loadregister with lowbyte of low time
 *
 * INTERNAL FUNCTION.\n
 * The function vpPwm_lowL() moves the lowbyte of the low time into the
 * appropiate loadregister. The vpPwm address is passed via _MainTemp,
 * wheras the lowbyte for the low time is passed in W.
 *
 * \param W Low byte of low time.
 */
static void vpPwm_lowL(char W)
{
#pragma update_FSR 0
  _MainTemp ^= W;
  W ^= _MainTemp;
  _MainTemp ^= W;
  FSR = W;
  while (vpPwm_updated) ;
  W = _MainTemp;
  vpPwm_newLow.low8 = W;
  W = FSR;
  _MainTemp = W;
#pragma update_FSR 1
}

/**
 * \brief Setup loadregister with highbyte of low time
 *
 * INTERNAL FUNCTION.\n
 * The function vpPwm_lowH() moves the highbyte of the low time into the
 * appropiate loadregister. The vpPwm address is passed via _MainTemp,
 * wheras the highbyte for the low time is passed in W.
 *
 * \param W Highbyte of low time.
 */
static void vpPwm_lowH(char W)
{
#pragma update_FSR 0
  _MainTemp ^= W;
  W ^= _MainTemp;
  _MainTemp ^= W;
  FSR = W;
  W = _MainTemp;
  vpPwm_newLow.high8 = W;
  vpPwm_newLow = -vpPwm_newLow; // -low
#pragma update_FSR 1
}

/**
 * \brief Set PWM low time, period preserved
 *
 * The function vpPwm_lowPeriod() sets the low time for the PWM signal and adjusts the high time
 * such that the new low time + high time equals the current low time + high time. The low time is
 * measured in isr cycles.
 *
 * \param pwm Address of vpPwm.
 * \param low Low time measured in isr cycles.
 */
#define vpPwm_lowPeriod(pwm,low) \
{ \
  _MainTemp = pwm; \
  W = (low) & 0xFF; \
  vpPwm_lowPeriodL(W); \
  W = (low) >> 8; \
  vpPwm_lowPeriodH(W); \
}

/**
 * \brief Setup loadregister with lowbyte of low time and calculate period
 *
 * INTERNAL FUNCTION.\n
 * The function vpPwm_lowPeriodL() moves the lowbyte of the low time into the
 * appropiate loadregister. The vpPwm address is passed via _MainTemp,
 * wheras the lowbyte for the low time is passed in W.
 *
 * \param W Lowbyte of low time.
 */
static void vpPwm_lowPeriodL(char W)
{
#pragma update_FSR 0
  _MainTemp ^= W;
  W ^= _MainTemp;
  _MainTemp ^= W;
  FSR = W;
  while (vpPwm_updated) ;
  vpPwm_newHigh += vpPwm_newLow; // -high + -low = -(low+high) = -period
  W = _MainTemp;
  vpPwm_newLow.low8 = W;
  W = FSR;
  _MainTemp = W;
#pragma update_FSR 1
}

/**
 * \brief Setup loadregister with highbyte of low time and adjust high time
 *
 * INTERNAL FUNCTION.\n
 * The function vpPwm_lowPeriodH() moves the highbyte of the low time into the
 * appropiate loadregister. The vpPwm address is passed via _MainTemp,
 * wheras the highbyte for the low time is passed in W.
 *
 * \param W Highbyte of low time.
 */
static void vpPwm_lowPeriodH(char W)
{
#pragma update_FSR 0
  _MainTemp ^= W;
  W ^= _MainTemp;
  _MainTemp ^= W;
  FSR = W;
  W = _MainTemp;
  vpPwm_newLow.high8 = W;
  vpPwm_newHigh += vpPwm_newLow; // -period + low = -(period-low) = -high
  vpPwm_newLow = -vpPwm_newLow;  // -low
#pragma update_FSR 1
}

/**
 * \brief Set PWM high time
 *
 * The function vpPwm_high() sets the high time for the PWM signal. The high time is measured
 * in isr cycles. The PWM period time equals low time + high time.
 *
 * \param pwm Address of vpPwm.
 * \param high High time measured in isr cycles.
 */
#define vpPwm_high(pwm,high) \
{ \
  _MainTemp = pwm; \
  W = (high) & 0xFF; \
  vpPwm_highL(W); \
  W = (high) >> 8; \
  vpPwm_highH(W); \
}

/**
 * \brief Setup loadregister with lowbyte of high time
 *
 * INTERNAL FUNCTION.\n
 * The function vpPwm_highL() moves the lowbyte of the high time into the
 * appropiate loadregister. The vpPwm address is passed via _MainTemp,
 * wheras the lowbyte for the high time is passed in W.
 *
 * \param W Lowbyte of high time.
 */
static void vpPwm_highL(char W)
{
#pragma update_FSR 0
  _MainTemp ^= W;
  W ^= _MainTemp;
  _MainTemp ^= W;
  FSR = W;
  while (vpPwm_updated) ;
  W = _MainTemp;
  vpPwm_newHigh.low8 = W;
  W = FSR;
  _MainTemp = W;
#pragma update_FSR 1
}

/**
 * \brief Setup loadregister with highbyte of high time
 *
 * INTERNAL FUNCTION.\n
 * The function vpPwm_highH() moves the highbyte of the high time into the
 * appropiate loadregister. The vpPwm address is passed via _MainTemp,
 * wheras the highbyte for the high time is passed in W.
 *
 * \param W Highbyte of high time.
 */
static void vpPwm_highH(char W)
{
#pragma update_FSR 0
  _MainTemp ^= W;
  W ^= _MainTemp;
  _MainTemp ^= W;
  FSR = W;
  W = _MainTemp;
  vpPwm_newHigh.high8 = W;
  vpPwm_newHigh = -vpPwm_newHigh; // -high
#pragma update_FSR 1
}

/**
 * \brief Set PWM high time, period preserved
 *
 * The function vpPwm_highPeriod() sets the high time for the PWM signal and adjusts the low time
 * such that the new low time + high time equals the current low time + high time. The high time is
 * measured in isr cycles.
 *
 * \param pwm Address of vpPwm.
 * \param high High time measured in isr cycles.
 */
#define vpPwm_highPeriod(pwm,high) \
{ \
  _MainTemp = pwm; \
  W = (high) & 0xFF; \
  vpPwm_highPeriodL(W); \
  W = (high) >> 8; \
  vpPwm_highPeriodH(W); \
}

/**
 * \brief Setup loadregister with lowbyte of high time and calculate period
 *
 * INTERNAL FUNCTION.\n
 * The function vpPwm_highPeriodL() moves the lowbyte of the high time into the
 * appropiate loadregister. The vpPwm address is passed via _MainTemp,
 * wheras the lowbyte for the high time is passed in W.
 *
 * \param W Lowbyte of high time.
 */
static void vpPwm_highPeriodL(char W)
{
#pragma update_FSR 0
  _MainTemp ^= W;
  W ^= _MainTemp;
  _MainTemp ^= W;
  FSR = W;
  while (vpPwm_updated) ;
  vpPwm_newLow += vpPwm_newHigh; // -low + -high = -(low+high) = -period
  W = _MainTemp;
  vpPwm_newHigh.low8 = W;
  W = FSR;
  _MainTemp = W;
#pragma update_FSR 1
}

/**
 * \brief Setup loadregister with highbyte of high time and adjust low time
 *
 * INTERNAL FUNCTION.\n
 * The function vpPwm_highPeriodH() moves the highbyte of the high time into the
 * appropiate loadregister. The vpPwm address is passed via _MainTemp,
 * wheras the highbyte for the high time is passed in W.
 *
 * \param W Highbyte of high time.
 */
static void vpPwm_highPeriodH(char W)
{
#pragma update_FSR 0
  _MainTemp ^= W;
  W ^= _MainTemp;
  _MainTemp ^= W;
  FSR = W;
  W = _MainTemp;
  vpPwm_newHigh.high8 = W;
  vpPwm_newLow += vpPwm_newHigh; // -period + high = -(period-high) = -low
  vpPwm_newHigh = -vpPwm_newHigh; // -high
#pragma update_FSR 1
}

/**
 * \brief Update running registers from loadregisters
 *
 * The function vpPwm_update() sets a flag to indicate all loadregisters are setup properly.
 * The loadregisters are copied to the running registers on the next interrupt cycle.
 *
 * \param pwm vpPwm address.
 */
#define vpPwm_update(pwm) \
{ \
  FSR = pwm; \
#pragma update_FSR 0 \
  vpPwm_updated = 1; \
#pragma update_FSR 1 \
}

#pragma library 0
#endif
// End of file vpPwm.h
